package edu.harvard.mgh.purcell.gplink.data;

import org.w3c.dom.*;
import org.xml.sax.*;

import ch.ethz.ssh2.Connection;
//import ch.ethz.ssh2.SCPClient;
import ch.ethz.ssh2.ConnectionMonitor;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;

import javax.swing.JOptionPane;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import edu.harvard.mgh.purcell.gplink.mainGUI.*;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
/**
 * This contains the data for using our operation traking.
 * @author Kathe Todd-Brown
 *
 */
public class Project extends KeyWords{
	
	
	///////////////////////////////////////////////////////
	///////////////Non-Static Variables ///////////////////
	///////////////////////////////////////////////////////

	/**
	 * contains all the file and operations information
	 */
	private Document projectInfo;
	
	/**
	 * contains the root folder this is all in
	 */
	private File rootfolder;
	
	/**
	 * path to the project folder in the remote machine
	 */
	private String remotefolder;
	
	/**
	 * remote host
	 */
	private String host;
	
	/**
	 * remote user
	 */
	private String user;
	
	/**
	 * port for ssh connection
	 */
	private int port;

	/**
	 * is the session flagged as read only
	 */
	private boolean readonly;
		
	//TODO access conn only via getter/setter
	/**
	 * ssh connection
	 */
	private Connection conn;
	
	/**
	 * frame that we are interacting with the project with
	 */
	public MainFrame frame;
	
	/**
	 * automatically checks for new files
	 */
	public AutoUpdater gplinkLog;
	
	
	//private boolean hasBroken;
	
	private ConnectionMonitor connChecker = new ConnectionMonitor(){

		public void connectionLost(Throwable reason) {
			Project.this.connectionLost();
			//hasBroken = true;
		}
		
	};
	
	///////////////////////////////////////////////////////
	///////////////  Getter Functions  ////////////////////
	///////////////////////////////////////////////////////

	/*public boolean isConnBroken(){
		return hasBroken;
	}*/
	private String getUser(){
		return user;
	}
	
	public Connection getConn(){
		return conn;
	}
	
	public String getHost(){
		return host;
	}
	
	public int getPort(){
		return port;
	}
	
	public boolean getBrowse(){
		return readonly;
	}
	
	/**
	 * get the metafile version
	 * @return the version of the current project
	 */
	public String getVersion(){
		return projectInfo.getDocumentElement().getAttribute(KeyWords.VS_KEY);
	}
	
	/**
	 * get the user defined editor command
	 * @return a string identifying the current user
	 * defined editor
	 */
	public String getEditor(){
		return projectInfo.getDocumentElement().getAttribute(USER_EDITOR_KEY);
	}
	
	/**
	 * get the default editor associated with the os type
	 * @return a string contianing the default editor command
	 */
	public String getDefaultEditor(){
		if(System.getProperty("os.name").contains("window")
				||  System.getProperty("os.name").contains("Window")){
			//you use window's you poor poor baby
			return WINDOW_DEFAULT_EDITOR;
		} else if(System.getProperty("os.name").contains("mac")
				|| System.getProperty("os.name").contains("Mac")){
			return MAC_DEFAULT_EDITOR;
		} 	
		else
			return LINUX_DEFAULT_EDITOR;
	}
	
	/**
	 * return the current path of plink
	 * @return returns a string that contains the location of plink
	 */
	public String getPlinkPath(){
		return projectInfo.getDocumentElement().getAttribute(PLINK_PATH_KEY);
	}
	
	/**
	 * Get the full haploview path name
	 * @return a string contianing the location of haploview
	 */
	public String getHaploviewPath(){
		return projectInfo.getDocumentElement().getAttribute(HAPLOVIEW_PATH_KEY);
	}
	
	/**
	 * Get the string that is appended to haploview java launch command
	 * in this project
	 * @return return a string that should be appended to the haploview
	 * launching command
	 */
	public String getHaploviewAppend(){
		return projectInfo.getDocumentElement().getAttribute(HAPLOVIEW_APPEND);
	}
	
	/**
	 * look at the root
	 * @return the root directory folder
	 */
	public File getRoot(){
		return rootfolder;
	}
	
	/**
	 * get the string that defines the remote project file
	 * @return the path of the remote project directory
	 */
	public String getRemoteFile(){
		return remotefolder;
	}
	
	
	
	/**
	 * Get any prefix to that belongs before the plink command is passed
	 * @return a string that is the plink prefix
	 */
	public String getPlinkPrefix(){
		return projectInfo.getDocumentElement().getAttribute(PLINK_PREFIX);
	}
	
	/**
	 * get the number of operations in a project.
	 * @return the numbre of operations ever created in this project
	 */
	//public int getNumOp(){
	//	return Integer.parseInt(projectInfo.getDocumentElement().getAttribute(EOP_KEY));
	//}
	
	/**
	 * get the name of a project
	 * @return a String which is the name of the project
	 */
	public String getProjectName(){
		if (projectInfo == null)
			return "";
		return projectInfo.getDocumentElement().getAttribute(NAME_KEY);
	}	
	///////////////////////////////////////////////////////
	///////////////  Setter Functions  ////////////////////
	///////////////////////////////////////////////////////

	/**
	 * Set the browse flag
	 * @param browse a boolean that states the if you want the session
	 * to be browse only (true) or fully accessed (false)
	 */
	public void setBrowse(boolean browse){
		if(frame != null
				&& frame.menuStuff != null){
			frame.menuStuff.setMenuLock(!browse);
		}
		readonly = browse;
	}
	
	/**
	 * Set the current host.
	 * @param h a string discribing the host name on a remote
	 * session
	 */
	void setHost(String h){
		host = h;
	}
	
	
	/**
	 * Set the current user.
	 * @param u a string discribing the user name on the remote
	 * session
	 */
	void setUser(String u){
		user = u;
	}
	
	/**
	 * Set the port number.
	 * @param p an int specifying the port number for a remote ssh
	 * connection
	 */
	void setPort(int p){
		port = p;
	}
	
	/**
	 * Set the Connection for the project.
	 * @param c the connection that will be used in the 
	 * remote connection
	 */
	public void setConnection(Connection c){
		conn = c;
		//hasBroken = false;
		conn.addConnectionMonitor(connChecker);
	}

	/**
	 * Set the local project file without initalizing the
	 * entire project.
	 * @param given file that makes the local project file
	 */
	public void setLocalRoot(File given){
		rootfolder = given;
	}
	
	/**
	 * set the path of plink
	 * @param p absolute path name of the plink exicutable
	 */
	public void setPlinkPath(String p){
		projectInfo.getDocumentElement().setAttribute(PLINK_PATH_KEY, p);
	}
	
	/**
	 * Set the haploview path
	 * @param p absolute path name of the haploview .jar file
	 */
	public void setHaploviewPath(String p){
		projectInfo.getDocumentElement().setAttribute(HAPLOVIEW_PATH_KEY, p);
	}
	
	/**
	 * Set an append to the command call to haploview
	 * @param s a string to append to haploview call
	 */
	public void setHaploviewAppend(String s){
		projectInfo.getDocumentElement().setAttribute(HAPLOVIEW_APPEND, s);
	}
	
	/**
	 * Set a preference to the plink command call
	 * @param p a string to prefix to the plink call
	 */
	public void setPlinkPrefix(String p){
		projectInfo.getDocumentElement().setAttribute(PLINK_PREFIX, p);
	}
	
	/**
	 * Set the user defined editor. This is a command to open an editor
	 * @param s an editor command
	 */
	public void setEditor(String s){
		projectInfo.getDocumentElement().setAttribute(USER_EDITOR_KEY, s);
	}
	
	/**
	 * If a connection is open then reset the remote project file.
	 * @param given an absolute path to the remote project file
	 */
	public void setRemoteFile(String given){
		//System.out.println("Entering setRemoteFile()@Project.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());

		//check that the connection is there
		if( !given.equals("") 
				&& conn == null){
			new ErrorDialog(frame, "No connection is open therefor I can not reset the remote project file. \n" +
			"setRemoteFile@Project.java");
			return; 
		}
		
		//rest the remote project
		remotefolder = given;
		
		if(given.equals("")){
			//System.out.println("Exiting(1) setRemoteFile()@Project.java "
			//		+ (new GregorianCalendar()).getTimeInMillis());
			return;
		}
		
		/*
		 * Save the remote project configuration to local project
		 */
		Properties config = new Properties();
		
		try{
			config.setProperty(REMOTE_PROJECT_KEY, remotefolder);
			config.setProperty(USER_KEY, getUser());
			config.setProperty(HOST_KEY, getHost());
			config.setProperty(PORT_KEY, new Integer(getPort()).toString());
			config.store(new FileOutputStream(new File(getRoot(), REMOTE_FILE)), 
			"gPLINK remote configuration");
		} catch (IOException e) {
			new ErrorDialog(frame, "Error saving the remote configuration file. \n" +
			"setRemoteFile@Project.java");
		}

		//frame.menuStuff.setMenuLock(true);
		
		//System.out.println("Flag1 setRemoteFile()@Project.java "
		//				+ (new GregorianCalendar()).getTimeInMillis());
			
		//load the project in the new remote folder
		loadData();
		//System.out.println("Flag2 setRemoteFile()@Project.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());

		//copy log and gplink files from the host
		//frame.updateViews();
		
		//if it's attached to a MainFrame
		if(frame != null){
			//and there's an active gplinkLog
			if(gplinkLog != null)
				//cancel the gplinkLog
				gplinkLog.cancel();
			//reset the gplinkLog
			gplinkLog = new AutoUpdater(this);
		} else {
			//otherwise set the log to null since there's
			//no need to update a null MainFrame
			gplinkLog = null;
		}
		//System.out.println("Exiting setRemoteFile()@Project.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());

	}
	
	/////////////////////////////////////////////////////
	/////////////////////Locking Functions ///////////////
	/////////////////////////////////////////////////////

	/**
	 * Check to see if the lock is avalable for this project.
	 * @return true if the lock is avaliable
	 */
	public boolean avalibleLock(){
		
		if(getBrowse() == true)
			return false;
		
		/*
		 * get the lock from the remote
		 */
		if(isRemote()){
			
			try {
				Session sess = conn.openSession();
				sess.execCommand("less " + GeneralUtil.makeSpacesHard(getRemoteFile() 
						+ "/" + LOCK_FILE));
				InputStream stdout = new StreamGobbler(sess.getStdout());
				BufferedReader br = new BufferedReader(new InputStreamReader(stdout));
				String remoteLock = br.readLine();
				sess.close();
				if(remoteLock != null
						&& !remoteLock.equals(LOCKED_CODE) 
						&& remoteLock.contains(LOCKED_BASE))
					return false;
				
			} catch (IOException e){
				new ErrorDialog(frame, "Error trying to read the remote lock");
				//connectionLost();
			}
			
		}
		/*
		 * Ensure that the files exist
		 */
		try {
			//if the file doesn't exist
			new File(getRoot(), LOCK_FILE).createNewFile();
		} catch (IOException e) {
			new ErrorDialog(frame, "Error ensuring local file exists. (2) \n " +
					"avaliableLock@Project.java");
		}
		
		/*
		 * read in the lock setting
		 */
		String localLock = "";
		try{
			/*
			 * read the local lock
			 */
			BufferedReader in = new BufferedReader(
					new FileReader(new File(getRoot(), LOCK_FILE)));
			//the lock code is on the first line
			localLock = in.readLine();
			in.close();
		}catch(IOException e){
			new ErrorDialog(frame, "Error trying to read in local lock.\n" +
					"avalibleLock@Project.java");
		}
		
		//does the lock either unclaimed or claimed by the current project
		if (localLock != null
				&& !localLock.equals(LOCKED_CODE) 
				&& localLock.contains(LOCKED_BASE))
			return false;
		else
			return true;
		
		
	}
	
	/**
	 * Check to see if the project currently has the lock
	 * @return true if the lock key equals that of the current project
	 */
	public boolean hasMetaLock(){
		
		if(getBrowse() == true)
			return false;
		
		/*
		 * use less to look at the data in the file
		 */
		if( isRemote() ){
			try{
				Session sess = conn.openSession();
				sess.execCommand("less " + GeneralUtil.makeSpacesHard(getRemoteFile() 
						+ "/" + LOCK_FILE));
				InputStream stdout = new StreamGobbler(sess.getStdout());
				BufferedReader br = new BufferedReader(new InputStreamReader(stdout));
				String remoteLock = br.readLine();
				sess.close();
				if(remoteLock == null
						|| !remoteLock.equals(LOCKED_CODE))
					return false;
				
			} catch (IOException e){
				new ErrorDialog(frame, "Error trying to read the remote lock");
				//connectionLost();
			}
		}
		
		/*
		 * Make sure that there is a file to look at
		 */
		try { 
			if(new File(getRoot(), LOCK_FILE).createNewFile()){
				//unlockMeta();
				return false;
			}
		} catch (IOException e1) { 
			new ErrorDialog(null, "Error trying to create the local lock.\n" +
				"hasMetaLock@Project.java");
		}
		
		/*
		 * read in the lock and see if they match the current
		 */
		String localLock ="";
		try{
			BufferedReader in = new BufferedReader(
					new FileReader(new File(getRoot(), LOCK_FILE)));
			localLock = in.readLine();
			in.close();
		}catch(IOException e){
			new ErrorDialog(null, "Error trying to read local project lock.\n" +
					"hasMetaLock@Project.java");
		}
		
		return localLock != null 
				&& LOCKED_CODE.equals(localLock);
	}

	
	/**
	 * Lock the project file(s) so that no other instance of gPLINK will conflict.
	 * If the project is a remote project then lock both the local and remote files.
	 *
	 */
	public void lockMeta(){
		
		/*
		 * try to lock the local project file
		 */
		try {
			BufferedWriter out = new BufferedWriter(
					new FileWriter(new File(getRoot(), LOCK_FILE)));
			out.write(LOCKED_CODE);
			out.newLine();
			out.close();
			//activate the appropreate menu items
			if(frame!=null)
				frame.menuStuff.setMenuLock(true);
		} catch (IOException e) {
			new ErrorDialog(frame, "Error writing the lock file locally. \n" +
			"lockMeta@Project.java");
			//eprintStackTrace();
		}
		
		/*
		 *get the lock from the remote
		 */
		if( isRemote()){
			Session sess;
			try {
			sess = conn.openSession();
			sess.execCommand("echo \"" + LOCKED_CODE + "\" > " 
					+ GeneralUtil.makeSpacesHard(getRemoteFile()+"/" +LOCK_FILE));
			sess.close();
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to write the lock file remotely.\n" +
				"lockMeta:Project.java");
				//connectionLost();
			}
			
		}
		
	}
	/**
	 * Unlock the project file(s) so that no other instance of gPLINK will confict.
	 * If the project is a remote project then unlock both the local and remote files.
	 *
	 */
	public void unlockMeta(){
		
		/*
		 * try to lock the local project file
		 */
		try {
			BufferedWriter out = new BufferedWriter(
					new FileWriter(new File(getRoot(), LOCK_FILE)));
			out.write(UNLOCKED_CODE);
			out.newLine();
			out.close();
			//deactivate the appropreate menu items
			if(frame != null)
				frame.menuStuff.setMenuLock(false);
		} catch (IOException e) {
			new ErrorDialog(frame, "Error writing the lock file locally. \n" +
			"unlockMeta@Project.java");
			//eprintStackTrace();
		}
		
		/*
		 *get the lock from the remote
		 */
		if( isRemote()){
			Session sess;
			try {
			sess = conn.openSession();
			sess.execCommand("echo \"" + UNLOCKED_CODE + "\" > " 
					+ GeneralUtil.makeSpacesHard(getRemoteFile()+"/" + LOCK_FILE));
			sess.close();
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to write the lock file remotely.\n" +
				"unlockMeta:Project.java");
				//connectionLost();
			}
		}
	}
	
	
	
	///////////////////////////////////////////////////////
	/////////////// Remote Functions   ////////////////////
	///////////////////////////////////////////////////////
	
	
	/**
	 * Close the remote connection and clear the remote project file parameter.
	 */
	public void closeRemote(){
		//if the connection is not null then close it
		if(conn != null)
			conn.close();
		//make sure the connection is null
		conn = null;
		//clear the remote project file parameter
		setRemoteFile("");
		setBrowse(frame.menuStuff.browseOnly.isSelected());
		//setRemoteRoot("");
	}
	
	
	public boolean connectionLost(){
		gplinkLog.stop();
		int choice = JOptionPane.showConfirmDialog(frame, 
			"There is a problem with the connection.\n" +
			"Would you like try to reconnect?",
			"Connection Error",
		    JOptionPane.YES_NO_OPTION);
		
		if(choice == JOptionPane.YES_OPTION){
			String local = getRoot().getAbsolutePath();
			String remote = getRemoteFile();
			String tempHost = getHost();
			String tempUser = getUser();
			int tempPort = getPort();
			//closeRemote();
			closeOldProject();
			setLocalRoot(new File(local));
			new ProjectDialog(frame, local, remote, tempHost, 
					tempUser, tempPort);
			return true;
		}else{
			setBrowse(true);
			conn = null;
			//if(frame !=null)
			//	frame.updateViews();
			return false;
		}
	}
	
	public boolean isflaggedRemote(){
		if(getRoot() == null 
				|| new File(getRoot(), REMOTE_FILE).exists() == false)
			return false;
		else
			return true;
	}
	
	/**
	 * A project is remote if it has a non-empty local project folder
	 * a non-null connection and has a config file for a remote project
	 */
	//public boolean isRemote(boolean checkConn){
	public boolean isRemote(){
		if(getRoot() == null 
				|| conn == null
				|| new File(getRoot(), REMOTE_FILE).exists() == false)
			return false;
		else
			return true;		
	}
	
	//////////////////////////////////////////////////
	////////////////Manipulate Project Data //////////
	//////////////////////////////////////////////////
	

	/**
	 * Set the root file we are working in and load the new data
	 * file that we find there. Checks also to see that the given
	 * file is a directory. Note that this local root must be set
	 * before the remote project can be set.
	 * @param f this is the folder that we are working from
	 */
	public void setRoot(File f){
		
		if(f == null){
			rootfolder = null;
//			and there's an active gplinkLog
			if(gplinkLog != null)
				//cancel the gplinkLog
				gplinkLog.cancel();
			return;
		}
			
		
		//if the connection is open we want close it
		if(conn != null)
			closeRemote();
//		set the folder
		rootfolder = f;	
		//load the project
		loadData();
		
		
		//check that it is a valid folder
		if(f.isDirectory() == true){
			//check that it's not write protected
			if(f.canWrite() == true){
				//if it's attached to a MainFrame
				if(frame != null){
					//and there's an active gplinkLog
					if(gplinkLog != null)
						//cancel the gplinkLog
						gplinkLog.cancel();
					//reset the gplinkLog
					gplinkLog = new AutoUpdater(this);
				} else {
					//otherwise set the log to null since there's
					//no need to update a null MainFrame
					gplinkLog = null;
				}
				
				//if the lock is not avaliable
				if(getBrowse() == false
					&& avalibleLock() == true){
				//the lock was available so let's take it
				lockMeta();
				if(frame != null)
					frame.menuStuff.setMenuLock(true);
				}
			}
		} else
			new ErrorDialog(frame,"Trying to set the working project folder " +
					"to a non-directory");
		//Done in the gplinkLog
		//frame.updateViews();
	}
	
	
	/**
	 * Get a list of files in the current project directory, local or remote.
	 * @return a vector of strings that hold the full path names of all the files
	 * in the project folder.
	 */
	public Vector <String> getFileList(){
		
		if(getRoot() == null){
			return new Vector<String>();
		}
		
		Vector <String> ans = new Vector<String>();
		
		if(isRemote() == true){
			try{
			Session sess = conn.openSession();
			sess.execCommand("ls " + GeneralUtil.makeSpacesHard(remotefolder));
			InputStream stdout = new StreamGobbler(sess.getStdout());
			BufferedReader br = new BufferedReader(new InputStreamReader(stdout));
			String line = br.readLine();
			while (line!= null)
			{
				ans.add(getRemoteFile()+"/"+line);
				line = br.readLine();
			}
			sess.close();
		} catch (IOException e) {
			new ErrorDialog(frame, "Error trying to read files from remote folder.\n" +
					"getFileList@Project.java");
		}
		}else{
			File [] temp = getRoot().listFiles();
			for(int i = 0; i < temp.length; i ++){
				ans.add(temp[i].getAbsolutePath());
			}
		}
		return ans;
	
	}
	
	/**
	 * get the command line for a given operation
	 * @param name the name of the operation you are looking for
	 * @return the command line attached to that operation
	 */
	public String getOpCommand(String name){
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
			if(index.getNodeName().equals(OP_KEY)){
				//check to see if it has the right name
				if(((Element)index).getAttribute(NAME_KEY).equals(name))
					return ((Element)index).getAttribute(CLINE_KEY);
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
		
		return null;
	}
	
	/**
	 * get the discription of an operation based on a given name
	 * @param name a string containing the name of the operation
	 * @return a string contianing the discription associated with
	 * the operation
	 */
	public String getOpDiscription(String name){
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type (text, entity or operation)
			if(index.getNodeName().equals(OP_KEY)){
				//check to see if it has the right file/operation name
				if(((Element)index).getAttribute(NAME_KEY).equals(name)){
					return index.getFirstChild().getNodeValue();
				}
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
		
		return null;
	}
	
	/**
	 * get a list of all the operation names in the project
	 * @return a vector of all operations names this will be
	 * empty if there are no operations
	 */
	public Vector<String> getAllOpName(){
		//if we don't have any document initalized then logically we shouldn't
		//...return any operations
		if (projectInfo == null)
			return null;
		//create an answer for us to return
		Vector<String> ans = new Vector<String>();
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you fall off
		while(index != null){
			//checks to see if we are at the right type (text, entity or operation)
			if(index.getNodeName().equals(OP_KEY)){
				//grab the name, add it to your answer
				ans.add(((Element) index).getAttribute(NAME_KEY));
			}//end if
			//go to the next sibling
			index = index.getNextSibling();
		}//end while
		//return the ans
		return ans;
	}
	
	/**
	 * returns a list of the files used to construct a given operation
	 * and their corrisponding discriptions.
	 * @param opName the operation name that you are looking for
	 * @param inOut flag that looks at either the INFILE or OUTFILE of
	 * the operation
	 * @return a list of file nodes and text (used form XML formating) nodes
	 */
	public String [] [] getOpFiles(String opName, String inOut){
		String [][] ans = null;
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you fall off
		while(index != null){
			//checks to see if we are at the right type 
			if(index.getNodeName().equals(OP_KEY)){
				//we know that all the operations are elements
				Element temp = (Element) index;
				//check the name
				if(temp.getAttribute(NAME_KEY).equals(opName)){
					//go though the children nodes
					Node index2 = index.getFirstChild();
					while(index2 != null){
						//find the infile/outfile node
						if(index2.getNodeName().equals(inOut)){
							//get the children some of which will be text nodes
							//and some of which will be file nodes
							//int numFiles = Integer.parseInt(
							//		((Element)index2).getAttribute(NUM_FILE_KEY));
							//keep track of the index we are on so that we
							//...can populate the array
							int curIndex = 0;
							//set the answer finally
							Vector <String> fileNames = new Vector<String>();
							Vector <String> fileDisc = new Vector<String>();
							//ans = new String [2] [numFiles];
							//go through the childre nodes of the file node
							Node index3 = index2.getFirstChild();
							while (index3 != null){
								//make sure that it is a file and 
								//...not a text node
								if(index3.getNodeName().equals(FILE_KEY)){
									//ans[0][curIndex] = 
									fileNames.add(
										((Element)index3)
										.getAttribute(NAME_KEY));
									//ans[1][curIndex] = 
									fileDisc.add(
										index3.getTextContent());
									curIndex ++;
								}
								//get the next possible file node
								index3 = index3.getNextSibling();
							}
							ans = new String[2][fileNames.size()];
							for(int i = 0; i < ans[0].length; i ++){
								ans[0][i] = fileNames.get(i);
								ans[1][i] = fileDisc.get(i);
							}
						
							
							//we went though all the files and can now quit
							return ans;
						}//end if: outfile/infile name check
						
						//go to the next child node of our operation
						index2 = index2.getNextSibling();
					}//end while: going though all second tier children
					
					//we checked all decendents of our matching operation
					return ans;
				}//end if: operation name check
			}//end if: operation type check
			
			//go to the next sibling
			index = index.getNextSibling();
		}//go though all first tier children
		
		return ans;
	}

	
	/**
	 * construct the project by either settting&reading the root directory or
	 * setting all the internal info to null.
	 * @param f diretory that we wish to point the root to
	 */
	public Project(File f, MainFrame mf){
		setBrowse(false);
		frame = mf;
		
		
		
		//if the project is initalized with info
		if (f != null)
			//set the root folder we are working in
			setRoot(f);
		else{
			//set everything to null
			rootfolder = null;
			projectInfo = null;
		}
		
		
	}

	/**
	 * Save, unlock and set the root file to null.
	 *
	 */
	public void closeOldProject(){
		if(gplinkLog != null){
			gplinkLog.cancel();
			gplinkLog = null;
		}
		
		if(projectInfo != null
				&& getRoot() != null){
			writeProjectInfo(true, false, true);
			
			if(hasMetaLock())
				unlockMeta();
			
			projectInfo = null;
		}	
		
		frame.folderViewer.clearViewer();
		
		setBrowse(frame.menuStuff.browseOnly.isSelected());
			
		closeRemote();
		
		setRoot(null);
		
	}
	
	/**
	 * Update all the output files in the project
	 *
	 */
	public void updateFiles(){
		updateFiles(false);
	}
	
	public void updateFiles(boolean forceUpdate){
		
		if(projectInfo == null)
			return;
		
		Vector <String> allLogRoots = new Vector <String>();
		Vector <String> allFiles = getFileList();
		
		for(String files: allFiles){
			if(files.endsWith(".log") 
					|| files.endsWith(OP_STATUS_EXT)){
				String fileName = GeneralUtil.fileName(files);
				if(files.endsWith(".log")){
					allLogRoots.add(fileName.substring(0, fileName.length()-4));
				}
				if( this.isRemote() 
						&& ( ! new File(getRoot(), fileName).exists()
						|| forceUpdate == true)){
					DownloadFile getFiles = new DownloadFile(new String [] {GeneralUtil.makeSpacesHard(files)},
							(getRoot().getAbsolutePath()), getConn(), frame);
					getFiles.start();
					
					/*SCPClient scopy = new SCPClient(conn);
					try {
						scopy.get(GeneralUtil.makeSpacesHard(files),
								(getRoot().getAbsolutePath()));
					} catch (IOException e) {
						new ErrorDialog(frame, "Error trying to copy log & gplink files.\n" +
							"Copying ["	+ files + "] to [" 
							+ getRoot().getAbsolutePath() + "] \n" +
							"updateFiles:Project.java");
						
					}*/
					
				}
			}
		}
		
//		start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
			if(index.getNodeName().equals(OP_KEY)){
				Node index2 = index.getFirstChild();
				
				//go to the Outfiles node
				while(index2 != null
						&& (index2.getNodeName().equals(OUTFILE_KEY) == false )){
					index2 = index2.getNextSibling();
				}
				
				if(index2 == null){
					System.err.println("can't find outfile node " +
							"while updatingFiles");
					return;
				}
					
				
				String opName = ((Element) index).getAttribute(NAME_KEY);
				String [][] outFiles = getOpFiles(opName, OUTFILE_KEY);
				
				String rootName;
				
				
				
				//get a list of all files that have the same root
				rootName = opName;
				
				//File [] allFiles = rootfolder.listFiles();
				
				HashSet<String> filteredFiles = new HashSet <String>();
				
				for(String file: allFiles){
					if((isflaggedRemote() && file.contains("/" + rootName + "."))
							||(!isflaggedRemote() && file.contains(File.separator + rootName + ".")))
					{
						filteredFiles.add(file);
					}
				}
				
				Vector<String> temp = new Vector<String>();
				for(String file:filteredFiles){
//				double check that it doesn't match another
					for(String root: allLogRoots){
						if(!opName.equals(root)
								&& root.contains(opName)
								&& new File(file).getName().startsWith(root+"."))
							temp.add(file);
					}
				}
				
				for(String file:temp){
					filteredFiles.remove(file);
				}
				
				//we start with one file
				if(outFiles[0].length >= 1){
					//compare the two lists
					for(int i = 0; i < outFiles[0].length; i ++){
						if(filteredFiles.contains(outFiles[0][i]))
							filteredFiles.remove(outFiles[0][i]);
					}
				} else {
					System.err.println("Problem updating files.");
				}
				//add the new files
				for(String s: filteredFiles){
					addFile(index2, s, "");
				}
				
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
		
		return;
	}
	
	/**
	 * load the data from the log file if it exists. If it does not exist
	 * then create a head for the project (note that there is no call to write
	 * to the log file in this operation).
	 *
	 */
	@SuppressWarnings("static-access")
	private void loadData(){
		//System.out.println("Entering loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		//double check that the rootfile is not null
		if(rootfolder == null){
			new ErrorDialog(frame, "Error trying to load project data; root folder set to null.\n" +
					"loadData@Project.java");
			//System.err.println("loadData()@Project.java:Project needs to have the root folder set. No data loaded");
			return;
		}
		//create the whole file path of the log file
		File workinglog = new File(rootfolder.getAbsolutePath(), LOG_FILE);
		
		
		//use xml document builders to create a new document from the xml
		//...log file
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		DocumentBuilder parser = null;
		
		Properties localConfig = new Properties();
		
		//System.out.println("Flag1 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		//only stays this setting if you are in a remote folder
		File configFile = new File(System.getProperty("user.dir")
				+ File.separator + GLOBAL_CONFIG_FILE);
		
		try{
			//System.out.println("loadData@Project.java The config file is: " + configFile.getAbsolutePath());
			localConfig.load(new FileInputStream(configFile));
			
		}catch(Exception e1){
			//System.out.println("loadData@Project.java The config file is: " + configFile.getAbsolutePath());
			configFile = new File(System.getProperty("user.home")
					+ File.separator + GLOBAL_CONFIG_FILE);
			try{
				localConfig.load(new FileInputStream(configFile));
			}catch(Exception e2){
				localConfig.setProperty(PLINK_PATH_KEY, "plink");
				localConfig.setProperty(PLINK_PREFIX, "");
				localConfig.setProperty(HAPLOVIEW_PATH_KEY, "");
				localConfig.setProperty(HAPLOVIEW_APPEND, "");
				localConfig.setProperty(USER_EDITOR_KEY, "");
				localConfig.setProperty(PROJECT1, "");
				localConfig.setProperty(PROJECT2, "");
				localConfig.setProperty(PROJECT3, "");
				localConfig.setProperty(PROJECT4, "");
				localConfig.setProperty(BROWSE_KEY, FALSE);
			}
		}
		
		//System.out.println("Flag2 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		Properties remoteConfig = new Properties();
		if(isRemote()){
			DownloadFile getConfig = new DownloadFile("~/" + GLOBAL_CONFIG_FILE,
						getRoot().getAbsolutePath(), getConn(), frame, true);
			getConfig.start();
			try {
				getConfig.join();
			} catch (InterruptedException e) {
				
				new ErrorDialog(frame, "Error waiting for configuration download thread. \n" +
						"loadData@Project.java");
			}
			try{
			remoteConfig.load(new FileInputStream(
					new File(getRoot(), GLOBAL_CONFIG_FILE)));
			
			} catch (IOException e) {
				//it will all get taken care off later we just don't
				//set the remote properties "right" nothing breaks
				if(remoteConfig.getProperty(PLINK_PATH_KEY) == null)
					remoteConfig.setProperty(PLINK_PATH_KEY, "");
				if(remoteConfig.getProperty(PLINK_PREFIX) == null)
					remoteConfig.setProperty(PLINK_PREFIX, "");
			}
			/*try {
				SCPClient scopy = new SCPClient(conn);
				scopy.get("~/" + GLOBAL_CONFIG_FILE,
						getRoot().getAbsolutePath());
				remoteConfig.load(new FileInputStream(
						new File(getRoot(), GLOBAL_CONFIG_FILE)));
				
			} catch (IOException e) {
				//it will all get taken care off later we just don't
				//set the remote properties "right" nothing breaks
				if(remoteConfig.getProperty(PLINK_PATH_KEY) == null)
					remoteConfig.setProperty(PLINK_PATH_KEY, "");
				if(remoteConfig.getProperty(PLINK_PREFIX) == null)
					remoteConfig.setProperty(PLINK_PREFIX, "");
			}*/
			
		}
			
		if(localConfig.getProperty(PROJECT1) == null)
			localConfig.setProperty(PROJECT1, "");
		if(localConfig.getProperty(PROJECT2) == null)
			localConfig.setProperty(PROJECT2, "");
		if(localConfig.getProperty(PROJECT3) == null)
			localConfig.setProperty(PROJECT3, "");
		if(localConfig.getProperty(PROJECT4) == null)
			localConfig.setProperty(PROJECT4, "");
		if(localConfig.getProperty(HAPLOVIEW_PATH_KEY) == null)
			localConfig.setProperty(HAPLOVIEW_PATH_KEY, "");
		if(localConfig.getProperty(HAPLOVIEW_APPEND) == null)
			localConfig.setProperty(HAPLOVIEW_APPEND, "");
		if(localConfig.getProperty(PLINK_PATH_KEY) == null)
			localConfig.setProperty(PLINK_PATH_KEY, "plink");
		if(localConfig.getProperty(PLINK_PREFIX) == null)
			localConfig.setProperty(PLINK_PREFIX, "");
		if(localConfig.getProperty(USER_EDITOR_KEY) == null)
			localConfig.setProperty(USER_EDITOR_KEY, "");
		if(localConfig.getProperty(BROWSE_KEY) == null)
			localConfig.setProperty(BROWSE_KEY, FALSE);
		
		//System.out.println(config.get(PROJECT1));
		if(!localConfig.get(PROJECT1).equals(rootfolder.getAbsolutePath())){
			if(localConfig.get(PROJECT2).equals(getRoot().getAbsolutePath())){
				localConfig.setProperty(PROJECT2, localConfig.getProperty(PROJECT1));
			}else if(localConfig.get(PROJECT3).equals(getRoot().getAbsolutePath())){
				localConfig.setProperty(PROJECT3, 
						localConfig.getProperty(PROJECT2));
				localConfig.setProperty(PROJECT2, 
						localConfig.getProperty(PROJECT1));
			} else{
				localConfig.setProperty(PROJECT4, 
						localConfig.getProperty(PROJECT3));
				localConfig.setProperty(PROJECT3, 
						localConfig.getProperty(PROJECT2));
				localConfig.setProperty(PROJECT2, 
						localConfig.getProperty(PROJECT1));
			}
				
			localConfig.setProperty(PROJECT1, getRoot().getAbsolutePath());
			
			try{
				//System.out.println(configFile.getAbsolutePath());
				localConfig.store(new FileOutputStream(configFile), 
						"gPLINK global configuration");
			}catch(IOException e1){
				new ErrorDialog(frame, "Error storing configuration file.\n" +
						"loadData@Project.java");
				//return;
			}
			
			if(frame != null 
					&& frame.menuStuff != null)
				frame.menuStuff.setPreviousProjects(localConfig);
		}
		
		//System.out.println("Flag3 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		//copy the log file from the remote folder
		if(isRemote()){
			if(workinglog.exists() ){
				//delete any local copy of the metafile we have
				workinglog.delete();
			}
			
			DownloadFile getlog = new DownloadFile(GeneralUtil.makeSpacesHard(remotefolder 
						+ "/" + LOG_FILE), (getRoot().getAbsolutePath()), getConn(), frame, true);
			getlog.start();
			try {
				getlog.join();
			} catch (InterruptedException e) {
				new ErrorDialog(frame, "Error waiting for metafile download thread. \n" +
				"loadData@Project.java");
			}
			/*
			try {
				//copy the metafile
				SCPClient scopy = new SCPClient(conn);
				scopy.get(GeneralUtil.makeSpacesHard(remotefolder 
						+ "/" + LOG_FILE), (getRoot().getAbsolutePath()));
			} catch (IOException e) {
				//new ErrorDialog(frame, "Error trying to copy the log file for this" +
				//		" remote project.\n loadData@Project.java");
				//if the file isn't there then move on
				//***Don't even think about uncommenting this error message!
			}
			*/
		}
		
		//System.out.println("Flag4 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		try {
			parser = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			new ErrorDialog(frame, "Error trying to create a new document builder.\n" +
					"loadData@Project.java");
		}
		
		//System.out.println("Flag5 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		
		//take the remote parames that are appropreate
		Properties config = localConfig;
		if(isflaggedRemote()){
			config.setProperty(PLINK_PATH_KEY, 
					remoteConfig.getProperty(PLINK_PATH_KEY, "plink"));
			config.setProperty(PLINK_PREFIX, 
					remoteConfig.getProperty(PLINK_PREFIX, ""));
		}
		
		//System.out.println("Flag6 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		
		//check that the log file exists
		if(workinglog.exists() ){
			try {
				projectInfo = parser.parse(workinglog);
			} catch (SAXException e) {
				new ErrorDialog(frame, "Error trying to create project, " +
					"can not parse log file.\n Creating new " +
					"project, please import old operations.\n " +
					"loadData@Project.java");
				createNewProject(parser);
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to create project, " +
					"IOException.\n Creating new" +
					"project, please import old operations.\n " +
					"loadData@Project.java");
				createNewProject(parser);
			}
			projectInfo.getDocumentElement().setAttribute(VS_KEY, frame.VS);
		}else{
			createNewProject(parser);
		}
		
		//System.out.println("Flag7 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		
		setPlinkPath(config.getProperty(PLINK_PATH_KEY));
		setPlinkPrefix(config.getProperty(PLINK_PREFIX));
		setHaploviewPath(config.getProperty(HAPLOVIEW_PATH_KEY));
		setHaploviewAppend(config.getProperty(HAPLOVIEW_APPEND));
		setEditor(config.getProperty(USER_EDITOR_KEY));
		setBrowse(config.getProperty(BROWSE_KEY).equals(TRUE));
		
		//System.out.println("Flag8 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		
		if((!isflaggedRemote() && !projectInfo.getDocumentElement().getAttribute(HOME_KEY).equals(
				rootfolder.getAbsolutePath()))
				|| ( isflaggedRemote() && !projectInfo.getDocumentElement().getAttribute(HOME_KEY).equals(
						getRemoteFile())))
			replaceHome();
		
		//System.out.println("Flag9 loadData()@Project.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		
	}
	
	/**
	 * Create a new project from scratch.
	 * @param parser is a document builder
	 * configuration information for this project
	 */
	@SuppressWarnings("static-access")
	private void createNewProject(DocumentBuilder parser){

		//create a new document
		projectInfo = parser.newDocument();
		
		//put in the header information for the project but no file data
		projectInfo.createTextNode(String.format("%n", (Object []) null));
		Element root = 
			(Element) projectInfo.createElement("project");
		root.setAttribute(NAME_KEY, rootfolder.getName());
		root.setAttribute(VS_KEY, frame.VS);
		//root.setAttribute(EOP_KEY, "0");
		if(!isflaggedRemote()){
			root.setAttribute(HOME_KEY, rootfolder.getAbsolutePath());
		}else{
			root.setAttribute(HOME_KEY, getRemoteFile());
			
		}
		//apppend the root to the project
		projectInfo.appendChild(root);
		//pad the file with new lines to make it human readable
	    root.appendChild( projectInfo.createTextNode(String.format("%n", (Object []) null)));
	
	}
	
	/**
	 * Check with the user to see if they want the old home file instances
	 * replaces with the new one.
	 *
	 */
	public void replaceHome(){
		String oldHome = projectInfo.getDocumentElement().getAttribute(HOME_KEY);
	
		String newHome;
		String newHomeFolder;
		if(isflaggedRemote()){
			newHome = getRemoteFile();
			newHomeFolder = newHome + "/";
		}else{
			newHome = rootfolder.getAbsolutePath();
			newHomeFolder = newHome + File.separator;
		}
		
		int choice = JOptionPane.showConfirmDialog(null,
				"I detect a different path. " +
				"Do you want to replace all instances of: \n[" +
				oldHome +
				"] with [" + newHome + "]?\n" +
						"(Hint: if you have moved your folder select yes.)", 
				"Replace?", JOptionPane.YES_NO_OPTION);
		if(choice == JOptionPane.NO_OPTION)
			return;
		if(choice == JOptionPane.YES_OPTION){

			Node opIndex = projectInfo.getDocumentElement().getFirstChild();
			//go until you fall off
			while(opIndex != null){
				//checks to see if we are at the right type (text, entity or operation)
				if(opIndex.getNodeName().equals(OP_KEY)){
					Node groupIndex = opIndex.getFirstChild();
					while(groupIndex != null){
						if(groupIndex.getNodeName().equals(INFILE_KEY) 
								||groupIndex.getNodeName().equals(OUTFILE_KEY)){
							Node fileIndex = groupIndex.getFirstChild();
							while(fileIndex != null){
								if(fileIndex.getNodeName().equals(FILE_KEY)){
									String oldName = ((Element) fileIndex).getAttribute(NAME_KEY);
									String tempFileName = GeneralUtil.fileName(oldName);
									String newName = newHomeFolder + tempFileName;
									//if the old name starts with the oldhome and the length
									//...of the old file name is the sum of the oldHome file length
									// plus the short file name plus 1 (in other words there is just a file
									//...seperator between them) then we replace that with the new home
									//...folder
									if(oldName.startsWith(oldHome) && 
											(oldHome.length() + 1 + tempFileName.length() == oldName.length() ))
									((Element)fileIndex).setAttribute(NAME_KEY, newName);
								}
								fileIndex = fileIndex.getNextSibling();
							}
						}
					groupIndex = groupIndex.getNextSibling();
					}
				}//end if
				//go to the next sibling
				opIndex = opIndex.getNextSibling();
			}//end while
		
			projectInfo.getDocumentElement().setAttribute(HOME_KEY, newHome);
			
			writeProjectInfo(true, false, false);
		}
	}
	
	private Node getOperation(String name){
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
			if(index.getNodeName().equals(OP_KEY)){
				//check to see if it has the right name
				if(((Element)index).getAttribute(NAME_KEY).equals(name))
					return index;
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
		return null;
	}
	
	public void swapOp(String op1, String op2){
		
		Node firstNode = getOperation(op1);
		Node secondNode = getOperation(op2);
		
		//replace the seond node with the first and store the parentless old node
		secondNode = projectInfo.getDocumentElement()
						.replaceChild(firstNode.cloneNode(true), secondNode);
		
		projectInfo.getDocumentElement()
					.replaceChild(secondNode.cloneNode(true), firstNode);
		
	}
	
	/**
	 * add a file element to a given parent parent node with their discription.
	 * @param parent node that we are attaching the file-node too
	 * @param name set the name attribute of this file-node
	 * @param discription set the text of the file node
	 */
	void addFile(Node parent, String name, String discription){
		
		if(parent == null
				|| parent.getNodeType() != (Node.ELEMENT_NODE)){
			new ErrorDialog(frame, "We are trying to add a file to a " +
					"node which can not accept it.");
			return;
		}
		if(name == null){
			new ErrorDialog(frame, "Error trying to add a null operation name. \n" +
					"addFile@Project.java");
		}
		
		if(discription == null){
			discription = "";
		}
		
		//new file node
		Element newFile = (Element) projectInfo.createElement(FILE_KEY);
		//set the name attribute
		newFile.setAttribute(NAME_KEY, name);
		//set the text content
		//newFile.setTextContent(discription);
		newFile.appendChild(projectInfo.createTextNode(discription));
		//set up spacing
		parent.appendChild( projectInfo.createTextNode(String.format("%n",(Object[]) null)));
		//attach the file node
		parent.appendChild(newFile);
		//grab the file count in the parent
		//int num_files = Integer.parseInt(
		//		 ((Element)parent).getAttribute(NUM_FILE_KEY));
		//add this to the file count in the parent
		//((Element)parent).setAttribute(NUM_FILE_KEY, "" + (num_files +1));
				
	}
	
	/**
	 * Add a file to an operation.
	 * @param opName A string that is the operation's identifying name
	 * @param fileType A string that flags weather the file is input/output
	 * @param fileName A string that gives the file name
	 * @param fileDisc A string that holds the file discription
	 * @return a boolean that is  weather the file was added or not
	 */
	public boolean addFile(String opName, String fileType, String fileName, String fileDisc){
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
//			check to see if it has the right name
			if(index.getNodeName().equals(OP_KEY)
					&& opName.equals((((Element)index).getAttribute(NAME_KEY)))){
				//look at all the nodes in the operation
				Node index2 = index.getFirstChild();
				while(index2 != null){
					//look at either the infile or outfile list
					if(index2.getNodeName().equals(fileType)){
						addFile(index2, fileName, fileDisc);
						return true;
						}
					index2 = index2.getNextSibling();
					}
				}
			
			//go to the next sibling
			index = index.getNextSibling();
		}
		return false;
		
	}
	
	/**
	 * Remove a list of files from certain operations.
	 * @param operations Vector of operation names we wish to remove files from
	 * @param filename Vector of file names we wish to remove
	 */
	public void removeFile(Vector <String> operations, Vector <String> filename){
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
			if(index.getNodeName().equals(OP_KEY)){
				//check to see if it has the right name
				if(operations.contains(((Element)index).getAttribute(NAME_KEY))){
					//look at all the nodes in the operation
					Node index2 = index.getFirstChild();
					while(index2 != null){
						//look at either the infile or outfile list
						if(index2.getNodeName().equals(INFILE_KEY)
								|| index2.getNodeName().equals(OUTFILE_KEY)){
							//go through each file
							Node index3 = index2.getFirstChild();
							while(index3 != null){
								//check to see if the filename is on the nix list
								if(index3.getNodeName().equals(FILE_KEY)
										&& filename.contains(((Element)index3)
												.getAttribute(NAME_KEY))){
									//incrament the index
									Node temp = index3.getNextSibling();
									//remove the previous file entry
									index2.removeChild(index3);
									index3 = temp;
									//change the number of files here
									//int numFiles = Integer.parseInt(
									//		((Element)index2).getAttribute(NUM_FILE_KEY));
									//((Element)index2).setAttribute(NUM_FILE_KEY,
									//		(Integer.toString((numFiles -1))));
								}else{
									//just go on
									index3 = index3.getNextSibling();
								}
							}
						}
						index2 = index2.getNextSibling();
					}
				}
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
	}
	
	/**
	 * replace the discription of a file with a new discription
	 * @param parent parent node of the file we are looking for
	 * @param name name of the file we are looking for
	 * @param discription new discription of file
	 */
	void replaceFile(Node parent, String name, String discription){
		//dummy check the inputs
		if(parent == null
				|| parent.getNodeType() != (Node.ELEMENT_NODE)){
			System.err.println("We are trying to add a file to a " +
					"node which can not accept it.");
			return;
		}
		
		//go though the children of the parent node
		Node index = parent.getFirstChild();
		while(index != null){
			//check that we are at a file node with the right name
			if(index.getNodeName().equals(FILE_KEY)
					&& ((Element) index).getAttribute(NAME_KEY).equals(name)){
				//there might not be any children
				if(index.getFirstChild() != null)
					//reset the discription
					index.getFirstChild().setNodeValue(discription);
				else
					//add a new child with the discription
					index.appendChild(projectInfo.createTextNode(discription));
				return;
			}
			index = index.getNextSibling();
		}
		
		//System.err.println("Could not find the file to replace " +
		//		"the discription for.");
		return;
	}
	
	public String getFolderFileNode(String fileName){
		
//		dummy check the inputs
		if(projectInfo == null){
			System.err.println("We are trying to add a file to a " +
					"node which can not accept it.");
			return "";
		}
		
		fileName = GeneralUtil.fileName(fileName);
		
		//go though the children of the parent node
		Node index = projectInfo.getDocumentElement().getFirstChild();
		
		while(index != null){
			//go to the folder node
			if(index.getNodeName().equals(FOLDER_KEY)){
				Node index2 = index.getFirstChild();
				while(index2 != null){
//					find the fileName
					if(((Element)index2).getAttribute(SHORT_NAME_KEY)
							.equals(fileName)){
						return index2.getTextContent();
					}
					//set the discription
					//if you can't find the fileName
					//create an entry
					index2 = index2.getNextSibling();
				}
				
				return "";
			}
			index = index.getNextSibling();
		}
		
		return "";
	}
	
	public void setFolderFileNote(String fileName, String fileDisc){
//		dummy check the inputs
		if(projectInfo == null){
			System.err.println("We are trying to add a file to a " +
					"node which can not accept it.");
			return;
		}
		
		fileName = GeneralUtil.fileName(fileName);
		
		//go though the children of the parent node
		Node index = projectInfo.getDocumentElement().getFirstChild();
		
		while(index != null){
			//go to the folder node
			if(index.getNodeName().equals(FOLDER_KEY)){
				Node index2 = index.getFirstChild();
				while(index2 != null){
//					find the fileName
					if(((Element)index2).getAttribute(SHORT_NAME_KEY)
							.equals(fileName)){
						index2.replaceChild(projectInfo.createTextNode(fileDisc), 
								index2.getFirstChild());
						return;
					}
					//set the discription
					//if you can't find the fileName
					//create an entry
					index2 = index2.getNextSibling();
				}
				Element newFile = (Element) projectInfo.createElement(FILE_KEY);
				newFile.setAttribute(SHORT_NAME_KEY, fileName);
				newFile.appendChild(projectInfo.createTextNode(fileDisc));
				index.appendChild(newFile);
				return;
			}
			index = index.getNextSibling();
		}
		//we didn't find the node! so we add it
		//...new file node
		Element newFolder = (Element) projectInfo.createElement(FOLDER_KEY);
		projectInfo.getDocumentElement().appendChild( projectInfo.createTextNode(String.format("%n",(Object[]) null)));
		//attach the file node
		projectInfo.getDocumentElement().appendChild(newFolder);
		setFolderFileNote(fileName, fileDisc);
		return;
	}

	
	
	/**
	 * add an operation element to the project element.
	 * @param cline the actual command line that is sent plink
	 * @param name name that command for internal reference
	 * @param discription set the discription of that command
	 * @param input an array of names of the input files
	 * @param inputDis an array of discriptions corrisponding to the inptu files
	 * @param output an array of names of the output files
	 * @param outputDis an array of discriptions corrisponding to the output files
	 * @return return that operation element that you created incase 
	 * you want to do something with it
	 */
	public Element addOperation(String cline, String name, String discription,
						String [] input, String [] inputDis,
						String [] output, String [] outputDis){
		//create new element to hold operation
		Element newOp = (Element) projectInfo.createElement(OP_KEY);
		//set the appropreate attributes
		newOp.setAttribute(CLINE_KEY, cline);
		newOp.setAttribute(NAME_KEY, name);
		
		//set the text for the node as the discription
		newOp.setTextContent(discription);
		
		//create a new element to hold the infiles
		Element infiles = (Element) projectInfo.createElement(INFILE_KEY);
		//start with no file's
		//infiles.setAttribute(NUM_FILE_KEY, "0");
		//set a name key for the hell of it, don't know that it's actually
		//...used
		infiles.setAttribute(NAME_KEY, "Input Files");
		//go through and add each input file name and discription
		for(int i = 0; i < input.length; i ++){
			addFile(infiles, input[i], inputDis[i]);
		}
		//format so that humans can read the xml format
		newOp.appendChild( projectInfo.createTextNode(String.format("%n", (Object[]) null)));
		//add all the infiles to the new operation
		newOp.appendChild(infiles);
		
		//create the new element to hold the outfiles
		Element outfiles = (Element) projectInfo.createElement(OUTFILE_KEY);
		//outfiles.setAttribute(NUM_FILE_KEY, "0");
		outfiles.setAttribute(NAME_KEY, "Output Files");
		for(int i = 0; i < output.length; i ++){
			addFile(outfiles, output[i], outputDis[i]);
		}
		
		//format so that humans can read the xml format
		newOp.appendChild( projectInfo.createTextNode(String.format("%n",(Object[]) null)));
		newOp.appendChild(outfiles);
		//format so that humans can read the xml format
		newOp.appendChild( projectInfo.createTextNode(String.format("%n",(Object[]) null)));
		
		//get the number of operations ever created
		//int num_ever_op = Integer.parseInt(
		//	 projectInfo.getDocumentElement().getAttribute(EOP_KEY)) + 1;
		//add this to the operation count
		//projectInfo.getDocumentElement().setAttribute(EOP_KEY, "" + num_ever_op);
		
//		append the operation to the project
		projectInfo.getDocumentElement().appendChild(newOp);
		
		//writeProjectInfo(false, true);
		
		if(gplinkLog != null)
			gplinkLog.addOp(name);
		
		//return the element we have just created
		return newOp;
	}
	
	/**
	 * replace an old operation with a new one
	 * @param oldname name of the old operation
	 * @param cline new command line you want to add
	 * @param name new name of operation
	 * @param discription new discription
	 * @param input new input file names
	 * @param inputDis new input file discriptions
	 * @param output new output file names
	 * @param outputDis new output file discriptions
	 * @return return a pointer to the node that was modified
	 */
	public Node replaceOperation(String oldname, String cline, String name, 
						String discription, 
						String [] input, String [] inputDis,
						String [] output, String [] outputDis){

		//******Find the operation based on it's old name********
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//element that we are looking for
		Element renewOp = null;
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type and have the right name
			if(index.getNodeName().equals(OP_KEY)
				&& ((Element)index).getAttribute(NAME_KEY).equals(oldname)){
				renewOp = (Element)index;
				}
			//go to the next sibling
			if(index != null)
				index = index.getNextSibling();
		}
		
		if(renewOp == null){
			System.err.println("Can't find the operation to replace.");
			return null;
		}
		
		//********Reset the data***********
		//set the appropreate attributes
		renewOp.setAttribute(CLINE_KEY, cline);
		renewOp.setAttribute(NAME_KEY, name);
		
		//set the text for the node as the discription
		renewOp.getFirstChild().setNodeValue(discription);
		
		index = renewOp.getFirstChild();
		
		while(index != null){
			if(index.getNodeName().equals(INFILE_KEY)){
				for(int i = 0; i < input.length; i ++){
					replaceFile(index, input[i], inputDis[i]);
				}
			}
			if(index.getNodeName().equals(OUTFILE_KEY)){
				for(int i = 0; i < output.length; i ++){
					replaceFile(index, output[i], outputDis[i]);
				}
			}
			index = index.getNextSibling();
		}
		
		//return the element we have just replaced
		return renewOp;
		
	}
	
	/**
	 * delete an operation in the document by name
	 * @param name of the operation to be delete
	 * @return the operation that we just deleted incase we want it
	 */
	public Node removeOperation(String name){
		//start at the first child
		Node index = projectInfo.getDocumentElement().getFirstChild();
		//go until you are at the last child
		while(index != null){
			//checks to see if we are at the right type
			//...and that the name matches
			if(index.getNodeName().equals(OP_KEY)
					&& ((Element) index).getAttribute(NAME_KEY).equals(name)){
				//found it!
				return projectInfo.getDocumentElement().removeChild(index);
			}
			//go to the next sibling
			index = index.getNextSibling();
		}
		//we didn't find the operation
		return null;
	}
	
	
	/////////////////////////////////////////////////
	/////////////// Write Data //////////////////////
	/////////////////////////////////////////////////
	
	/**
	 * This writes the data that makes up the project 
	 * (files and operations in general)
	 * to either the log file (.metafile) and/or the output screen.
	 * @param fileFlag flags the print to file output
	 * @param screenFlag flags the print to screen output
	 * 
	 */
	public void writeProjectInfo(boolean fileFlag, 
			boolean screenFlag, boolean waitFlag){
		
		if(avalibleLock() == false
				&& hasMetaLock() == false 
				&& getBrowse() == false){
			int choice = JOptionPane.showConfirmDialog(frame, 
					"Some other instance of gPLINK has the lock. " +
					"Would you like to steal the " +
					"lock to save the project information?\n " +
					"If you don't then you will be unable to add any new operations. ", 
					"Steal?", JOptionPane.YES_NO_OPTION);
			if(choice == JOptionPane.CLOSED_OPTION
					|| choice == JOptionPane.NO_OPTION){
				setBrowse(true);
				frame.menuStuff.setMenuLock(!getBrowse());
			}
		}
		
		//System.out.println("writeProjectInfo@Project.java:lock check completed... moving on.");
		
		if(getBrowse() == true){
			return;
		}
		lockMeta();
		frame.menuStuff.setMenuLock(true);
		
//		 Use a Transformer for output
		   TransformerFactory tFactory = TransformerFactory.newInstance();
		   Transformer transformer = null;
	
			//configure the transformer
			try {
				transformer = tFactory.newTransformer();
			} catch (TransformerConfigurationException e1) {
				new ErrorDialog(frame, "Error trying to create transformer.\n" +
						"writeProjectInfo@Project.java");
			}
			//set the source of the data
			DOMSource source = new DOMSource(projectInfo);
			//check the flags to see where we should output
			if(screenFlag == true){
				//set the recieving stream to the output
				StreamResult result = new StreamResult(System.out);
				try {
					transformer.transform(source, result);
				} catch (TransformerException e) {
					new ErrorDialog(frame, "Error trying to transform source to" +
							"console.\n writeProjectInfo@Project.java");
				}
			}
			if(fileFlag == true){
				//set the recieving stream to the file specified by 
				//...rootfolder/logfile
				StreamResult ofile = null;
				try {
					ofile = new StreamResult(new FileWriter(
								new File(rootfolder, LOG_FILE)) );
				} catch (IOException e) {
					new ErrorDialog(frame, "Error trying to create file stream.\n" +
						"writeProjectInfo@Project.java");
				}
				try {
					transformer.transform(source, ofile);
				} catch (TransformerException e) {
					new ErrorDialog(frame, "Error trying to transform source to" +
					"file stream.\n writeProjectInfo@Project.java");
				}
			}
			
		if(isRemote()){
			String temp1 = getRoot().getAbsolutePath() + File.separator + LOG_FILE;
			String temp2 = GeneralUtil.makeSpacesHard(remotefolder);
			UploadFile uploader = new UploadFile(frame, temp1, temp2, getConn());
			uploader.start();
			if(waitFlag){
				try {
					uploader.join();
				} catch (InterruptedException e) {
					//fail quietly
				}
			}
			/*
			SCPClient scopy = new SCPClient(conn);
			try {
				//System.out.println("Project.writeProjectInfo(); scopy.put("+getRoot().getAbsolutePath() + File.separator + LOG_FILE+", "+
				//		GeneralUtil.makeSpacesHard(remotefolder)+", " + SCP_MODE + ")");
				
				scopy.put(temp1, temp2, SCP_MODE);
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to copy local meta file to remote" +
						" project.\n writeProjectInfo@Project.java");
			}*/
		}
		   
	}
	
	public void writeOperations(boolean fileFlag, boolean screenFlag){
		if(fileFlag == true){

			DateFormat dateFormat = new SimpleDateFormat("yyyy-MMM-dd-HH-mm-ss");
            java.util.Date date = new java.util.Date();
            File outputFile = new File(getRoot(), OP_SUM_FILE+dateFormat.format(date)+".html");
		
			try {
		        BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
		        out.write("<html><title>" + getProjectName()+"</title><body>");
		        out.write("<h2>"+getProjectName()+"</h2>");
		        out.write("<h3>Timestamp: " + dateFormat.format(date)+ "</h3>");
		        out.write("<h3>local project file location: " + getRoot() + "</h3>");
		        if(isflaggedRemote())
		        	out.write("<h3>remote project file location: " + getRemoteFile() + "</h3>");
		        out.newLine();
		        out.write("<ul>");
		        
		        for(String opName: getAllOpName()){
		        	out.write("<li>");
		        	out.write(opName+ ": " + getOpDiscription(opName));
		        	
		        	out.newLine();
		        	out.write("<ul>");
		        	out.write("<li>"+getOpCommand(opName)+"</li>");
		        	out.newLine();
		        			        	
		        	String [][] infiles = getOpFiles(opName, INFILE_KEY);
		        	out.write("<li>INFILES<ul>");
		        	out.newLine();
		        	for(int i = 0; i < infiles[0].length; i ++){
		        		out.write("<li>"+infiles[0][i] + ": " 
		        				+ getFolderFileNode(GeneralUtil.fileName(infiles[0][i])) 
		        				+ "; "+ infiles[1][i]+"</li>");
		        		out.newLine();
		        	}
		        	out.write("</ul></li>");
		        	
		        	String [][] outfiles = getOpFiles(opName, OUTFILE_KEY);
		        	out.write("<li>OUTFILES<ul>");
		        	out.newLine();
	        		for(int i = 0; i < outfiles[0].length; i ++){
	        			if(!outfiles[0][i].endsWith(OP_STATUS_EXT)){
			        		out.write("<li>"+outfiles[0][i] + ": " 
			        				+ getFolderFileNode(GeneralUtil.fileName(outfiles[0][i])) 
			        				+ "; "+ outfiles[1][i]+"</li>");
			        		out.newLine();
	        			}
		        	}
	        		out.write("</ul></li>");
	        		
	        		out.write("</ul>");
	        		out.write("</li>");
	        		out.newLine();
		        }
		        out.write("</ul>");
        		out.newLine();
        		out.write("</body></html>");
		        out.close();
		    } catch (IOException e) {
		    	new ErrorDialog(frame, "Error trying to write the operations \n" +
		    			"summary.\n writeOperations@Project.java");
		    }
		}
		if(screenFlag == true){
			System.out.println(getProjectName());
			System.out.println();
			for(String opName: getAllOpName()){
	        	System.out.print(opName+ ": " + getOpDiscription(opName));
	        	
	        	String [][] infiles = getOpFiles(opName, INFILE_KEY);
	        	System.out.println("INFILES");
	        	for(int i = 0; i < infiles[0].length; i ++){
	        		System.out.println(infiles[0][i] + ": " + infiles[1][i]);
	        	}
	        	
	        	String [][] outfiles = getOpFiles(opName, OUTFILE_KEY);
	        	System.out.println("OUTFILES");
        		for(int i = 0; i < outfiles[0].length; i ++){
        			System.out.println(outfiles[0][i] + ": " + outfiles[1][i]);
	        	}
        		System.out.println();
	        }
		}		
	}

}
