package edu.harvard.mgh.purcell.gplink.forms;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.harvard.mgh.purcell.gplink.base_form.*;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

@SuppressWarnings("serial")
public class FullAssocForm extends Forms {

	Forms.myCheckBox ciButton, adjustButton, cellButton;
	
	Forms.myRadioButton  mperm2Button, perm2Button, mGenButton, 
	mTrendButton, mDomButton, mRecButton, mBestButton;
	
	Forms.myJTextField ciText, cText, mperm2Text;
	
	static private Double CI_DEFAULT = new Double(0.95);
	static private Integer MPERM_DEFAULT = new Integer(1000);
	static private Integer CELL_DEFAULT = new Integer(5);

	public FullAssocForm(MainFrame mf) {
		super(mf, "Genotypic C/C association tests");
	}

	@Override
	protected void createBody() {

		ButtonGroup g1 = new ButtonGroup();
		g1.add(mBestButton);
		g1.add(mGenButton);
		g1.add(mDomButton);
		g1.add(mRecButton);
		g1.add(mTrendButton);
	
		ActionListener changePerm = new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				if(mperm2Button.isSelected())
					perm2Button.setSelected(false);
			}
			
		};
		ActionListener changeMperm = new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				if(perm2Button.isSelected())
					mperm2Button.setSelected(false);
			}
			
		};
		
		perm2Button.addActionListener(changeMperm);
		mperm2Button.addActionListener(changePerm);
		
		body.setLayout(new BoxLayout(body, BoxLayout.PAGE_AXIS));
	
		body.add(Box.createRigidArea(new Dimension(0,10)));
		
		body.add( new JLabel("Genotypic association tests (C/C) (--model)"));
		
		body.add(Box.createRigidArea(new Dimension(0,10)));
		
		JPanel cellPanel = new JPanel();
		cellPanel.add(cellButton);
		cellPanel.add(cText);
		cellPanel.setLayout(new BoxLayout(cellPanel, BoxLayout.X_AXIS));
		cellPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		body.add(cellPanel);
		
		JPanel ciPanel = new JPanel();
		ciPanel.add(ciButton);
		ciPanel.add(ciText);
		ciPanel.setLayout(new BoxLayout(ciPanel, BoxLayout.X_AXIS));
		ciPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		body.add(ciPanel);

		adjustButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		body.add(adjustButton);
		
		body.add(Box.createRigidArea(new Dimension(0,10)));
		
		JPanel constraints = new JPanel();
		constraints.setBorder(new TitledBorder("Permutation options"));
		constraints.setLayout(new BoxLayout(constraints, BoxLayout.Y_AXIS));
		constraints.setAlignmentX(Component.LEFT_ALIGNMENT);
			
		JPanel mp2Panel = new JPanel();
		mp2Panel.add(mperm2Button);
		mp2Panel.add(mperm2Text);
		mp2Panel.setLayout(new BoxLayout(mp2Panel, BoxLayout.X_AXIS));
		mp2Panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(mp2Panel);
		
		perm2Button.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(perm2Button);

		constraints.add(Box.createRigidArea(new Dimension(0,15)));
		
		mBestButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		mBestButton.setSelected(true);
		constraints.add(mBestButton);

		mGenButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(mGenButton);
		
		mTrendButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(mTrendButton);
		
		mDomButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(mDomButton);
		
		mRecButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		constraints.add(mRecButton);
		
		body.add(constraints);
		
	}

	@Override
	protected void initalize() {
		
		body = new JPanel();
		body.setLayout(new BoxLayout(body, BoxLayout.PAGE_AXIS));
				
		ciText = new myJTextField(CI_DEFAULT.toString(), 4);
		cText = new myJTextField(CELL_DEFAULT.toString(), 4);
		mperm2Text = new myJTextField(MPERM_DEFAULT.toString(), 4);
		
		cellButton = new myCheckBox("Minimum required observation per cell (--cell)", 
				cText); 
		ciButton = new myCheckBox("Confidence intervals (--ci)", ciText); 
		adjustButton = new myCheckBox("Adjusted p-values (--adjust)"); 
		mperm2Button = new myRadioButton("max(T) permutation mode (--mperm)", mperm2Text);
		perm2Button = new myRadioButton("Adaptive permutation mode (--perm)");
		mBestButton = new myRadioButton("Permute best test (default)");
		mGenButton = new myRadioButton("Permute genotypic test (--model-gen)");
		mTrendButton = new myRadioButton("Permute trend test (--model-trend)"); 
		mDomButton = new myRadioButton("Permute dominant test (--model-dom)");
		mRecButton = new myRadioButton("Permute recessive test (--model-rec)");
		
		
	}

	@Override
	protected boolean isBodyValid() {
		if( (cellButton.isSelected() && !cText.getText().matches("\\d*\\.?\\d+"))){
//			information.setText("Must enter a value associated with the --cell flag.");
			return false;
		}
		if(ciButton.isSelected() && !ciText.getText().matches("\\d*\\.?\\d+")){
//			information.setText("Must enter a value associated with the --ci flag.");
			return false;
		}
		if(mperm2Button.isSelected() && !mperm2Text.getText().matches("\\d*\\.?\\d+")){
//			information.setText("Must enter a value associated with the --mperm flag.");
			return false;
		}
		
		return true;
	}

	@Override
	protected String processBody() {
	
		String ans = "";
		
		ans +="--model";

		if(ciButton.isSelected())
			ans +=" --ci " + ciText.getText();
		if(mperm2Button.isSelected())
			ans += " --mperm " + mperm2Text.getText();
		if(perm2Button.isSelected())
			ans += " --perm";
		
		if(adjustButton.isSelected())
			ans += " --adjust";
		if(cellButton.isSelected())
			ans += " --cell " + cText.getText();

		if(mGenButton.isSelected())
			ans += " --model-gen";
		if(mTrendButton.isSelected())
			ans += " --model-trend";
		if(mDomButton.isSelected())
			ans += " --model-dom";
		if(mRecButton.isSelected())
			ans += " --model-rec";
		
		return ans;

	}

}
